; beacon-1.0.asm        First test prog          Oct 1999
; Turns the xmitter on, outputs a 3-tone pattern, turns xmitter off,
; repeats endlessly.
; (for interface development/testing)


;--- REGISTER DEFINITIONS ---------------------------------------------
.def   tick_count   = r0  ; The low-level tick counter, controls all timing
.def   tick_temp    = r1  ; Destructive copy of tick_count
.def   hp_temp      = r2  ;   "          "   "  half_period
.def   cyc_l_temp   = r3  ;   "          "   "  cycles_low
.def   cyc_h_temp   = r4  ;   "          "   "  cycles_high
.def   half_period  = r16 ; The half period time, in ticks
.def   cycles_l     = r17 ; Number of cycles, low byte
.def   cycles_h     = r18 ; Number of cycles, high byte
.def   temp         = r31 ; General temp storage


;--- PORT DEFINITIONS -------------------------------------------------
; Port D, bit 6 = audio modulation output
;         bit 5 = transmitter keying output,  0 = not transmitting
                ; pd76543210
.equ portd_defs = 0b01100000
.equ audio      = 6 ; single bit audio out line
.equ xmit       = 5 ; transmitter control bit (0 = transmit)
; Port B, (not used)


;--- INTERRUPT VECTORS ------------------------------------------------
; (None used.)
          .org  0
   RESET: rjmp  main   ; RESET vector
EXT_INT0: rjmp  INTR0  ; Int 0 handler
TIM0_OVF: rjmp  TIMR0  ; Timer 0 overflow int handler
ANA_COMP: rjmp  COMP   ; Analog comparator int handler


.include "../include/1200.def"
.include "tone_lib-0.1.asm"

;--- THE PROGRAM ------------------------------------------------------
main:   ldi   temp, portd_defs  ; Get port D direction definition
        out   DDRD, temp        ;     put it in DDRD
        ldi   temp, 15          ; Set up the
        mov   tick_count, temp  ;     tick count

again:  cbi   portd, xmit       ; Turn xmit bit on

        pause 0x09c4            ; t=0.25s
        sound 0x08e0, 0x0b      ;
        pause 0x1388            ; t=1s
        sound 0x0457, 0x09      ; t=2s, f=555.6Hz
        pause 0x1388            ; t=1s
        sound 0x02ca, 0x07      ; t=1s, f=714.3Hz
        pause 0x09c4            ; t=0.25s

        sbi portd, xmit         ; Turn off transmitter
        pause 0xc350            ; t=5s

        rjmp  again             ; Do forever.


;--- INTERRUPT SERVICE ROUTINES ---------------------------------------
; (there aren't any.)
INTR0:  reti
TIMR0:  reti
COMP:   reti


; EOF: beacon-1.0.asm
