; tone_lib-0.1.asm                        (09/1999)
; The start of a library of tone-generating routines...
; Doesn't use the timer/counter
;
;--- REGISTER DEFINITIONS
; These registers are for internal destructive use
; They do not get loaded immediate, so they can be assigned to r0-r15
;   tick_temp    Destructive copy of tick_count
;   hp_temp           "       "   "  half_period
;   cyc_l_temp        "       "   "  cycles_l
;   cyc_h_temp        "       "   "  cycles_h
;
; These registers are loaded immediate with parameters needed before 
; calling a subroutine in this lib. They are NOT destroyed upon return.
; These registers are oftern loaded immediate, so they should be 
; located in the range r16-r31 
;   tick_count   The low-level tick counter, controls all timing
;   half_period  The half period time, in ticks
;   cycles_l     Number of cycles, low byte
;   cycles_h     Number of cycles, high byte
;
;--- I/O DEFINITIONS
; Since this is single-bit sound, there's only 1 port bit, called 
; "audio" needed. It MUST be on portd.
;
;--- CALLABLE MACROS
; sound <number_of_cycles>, <half_period_length_in_ticks>
; pause <number_of_ticks>
;
; --- tone_lib description END ----------------------------------------
.NOLIST


.macro pause ; sets up registers and calls silence
             ldi   cycles_h, HIGH(@0)
             ldi   cycles_l, LOW(@0)
             rcall _silence
.endmacro


.macro sound ; sets up registers and calls tone
             ldi   cycles_h, HIGH(@0)
             ldi   cycles_l, LOW(@0)
             ldi   half_period, @1
             rcall _tone
.endmacro


_tone: ; The waveform generating subroutine ----------------------------
       ; Calls cpy_cyc_cntr and tick
          rcall _cpy_cyc_cntr         ; Make a copy of the cyc cntr
cyc_loop: mov   hp_temp, half_period  ; Make a copy of the period count
          cbi   portd, audio          ; Set audio bit LOW
on_loop:  rcall _tick                 ; Do tick, half_period number
          dec   hp_temp               ;    of times
          brne  on_loop               ;
          mov   hp_temp, half_period  ; Make another copy
          sbi   portd, audio          ; Set audio bit HIGH
off_loop: rcall _tick                 ; Leave it HIGH for x = 
          dec   hp_temp               ;    half_period number 
          brne  off_loop              ;    of ticks
          ; This next bit of code decrements the 16 bit cycle counter
          dec   cyc_l_temp            ; 1 cycle complete
          brne  cyc_loop              ; cont. if not 0
          tst   cyc_h_temp            ; is the high byte 0
          breq  skip_it1              ; skip the dec if it is
          dec   cyc_h_temp            ; 255 cycles done
skip_it1: brne  cyc_loop              ; Cont. if high byte not 0
          ret                         ; Done.


_silence: ; A subroutine used to generate spaces between tones ---------
         ; Calls cpy_cyc_cntr and tick
         ; SilenceTime = TickPeriod * CyclesCount
          rcall _cpy_cyc_cntr         ; Make a copy of the cyc cntr
mo_hush:  rcall _tick                 ; Waste some time
          dec   cyc_l_temp            ; 1 tick complete
          brne  mo_hush               ; cont. if not 0
          tst   cyc_h_temp            ; is the high byte 0
          breq  skip_it2              ; skip the dec if it is
          dec   cyc_h_temp            ; 255 ticks done
skip_it2: brne  mo_hush               ; Cont. if high byte not 0
          ret                         ; Done.



_tick: ; A subroutine used to generate small delays --------------------
       ; #cycles = 3*tick_count+4, tick_count = 132 for 100uS tick @ 4MHz
       ; tick_count = 15 for 100uS tick @ 500KHz
         mov  tick_temp, tick_count ; (1)   Make a copy of tick count
mo_tick: dec  tick_temp             ; (1)   Count down
         brne mo_tick               ; (1/2) Continue until tick=0
         ret                        ; (4)   Done.


_cpy_cyc_cntr: ; A subroutine used to make a copy of cycle counter ------
               ; (for internal use only)
          mov   cyc_h_temp, cycles_h  ; Make a destroyable copy
          mov   cyc_l_temp, cycles_l  ;    of the 16-bit cycle counter
          ret                         ; Return


;EOF: tone_lib-0.1.asm
.LIST
